@@ -4,7 +4,7 @@
 /* First off, code is include which follows the "include" declaration
 ** in the input file. */
 #include <stdio.h>
-#line 5 "./configparser.y"
+#line 5 "../../src/configparser.y"
 
 #include <assert.h>
 #include <stdio.h>
@@ -363,12 +363,14 @@ static char *yyTracePrompt = 0;
 ** Outputs:
 ** None.
 */
+#if 0
 void configparserTrace(FILE *TraceFILE, char *zTracePrompt){
   yyTraceFILE = TraceFILE;
   yyTracePrompt = zTracePrompt;
   if( yyTraceFILE==0 ) yyTracePrompt = 0;
   else if( yyTracePrompt==0 ) yyTraceFILE = 0;
 }
+#endif
 #endif /* NDEBUG */
 
 #ifndef NDEBUG
@@ -441,6 +443,7 @@ static const char *yyRuleName[] = {
 ** This function returns the symbolic name associated with a token
 ** value.
 */
+#if 0
 const char *configparserTokenName(int tokenType){
 #ifndef NDEBUG
   if( tokenType>0 && (size_t)tokenType<(sizeof(yyTokenName)/sizeof(yyTokenName[0])) ){
@@ -452,6 +455,7 @@ const char *configparserTokenName(int tokenType){
   return "";
 #endif
 }
+#endif
 
 /*
 ** This function allocates a new parser.
@@ -516,44 +520,44 @@ static void yy_destructor(YYCODETYPE yymajor, YYMINORTYPE *yypminor){
     case 23:
     case 24:
     case 25:
-#line 146 "./configparser.y"
+#line 146 "../../src/configparser.y"
 { buffer_free((yypminor->yy0)); }
-#line 521 "configparser.c"
+#line 525 "configparser.c"
       break;
     case 35:
-#line 137 "./configparser.y"
+#line 137 "../../src/configparser.y"
 { (yypminor->yy41)->free((yypminor->yy41)); }
-#line 526 "configparser.c"
+#line 530 "configparser.c"
       break;
     case 36:
-#line 138 "./configparser.y"
+#line 138 "../../src/configparser.y"
 { (yypminor->yy41)->free((yypminor->yy41)); }
-#line 531 "configparser.c"
+#line 535 "configparser.c"
       break;
     case 37:
-#line 139 "./configparser.y"
+#line 139 "../../src/configparser.y"
 { (yypminor->yy41)->free((yypminor->yy41)); }
-#line 536 "configparser.c"
+#line 540 "configparser.c"
       break;
     case 39:
-#line 140 "./configparser.y"
+#line 140 "../../src/configparser.y"
 { array_free((yypminor->yy40)); }
-#line 541 "configparser.c"
+#line 545 "configparser.c"
       break;
     case 40:
-#line 141 "./configparser.y"
+#line 141 "../../src/configparser.y"
 { array_free((yypminor->yy40)); }
-#line 546 "configparser.c"
+#line 550 "configparser.c"
       break;
     case 41:
-#line 142 "./configparser.y"
+#line 142 "../../src/configparser.y"
 { buffer_free((yypminor->yy43)); }
-#line 551 "configparser.c"
+#line 555 "configparser.c"
       break;
     case 42:
-#line 143 "./configparser.y"
+#line 143 "../../src/configparser.y"
 { buffer_free((yypminor->yy43)); }
-#line 556 "configparser.c"
+#line 560 "configparser.c"
       break;
     default:  break;   /* If no destructor action specified: do nothing */
   }
@@ -821,9 +825,9 @@ static void yy_reduce(
         /* No destructor defined for global */
         break;
       case 5:
-#line 119 "./configparser.y"
+#line 119 "../../src/configparser.y"
 { yymsp[-1].minor.yy78 = NULL; }
-#line 826 "configparser.c"
+#line 830 "configparser.c"
   yy_destructor(1,&yymsp[0].minor);
         break;
       case 6:
@@ -836,7 +840,7 @@ static void yy_reduce(
   yy_destructor(1,&yymsp[0].minor);
         break;
       case 9:
-#line 148 "./configparser.y"
+#line 148 "../../src/configparser.y"
 {
   if (ctx->ok) {
     buffer_copy_string_buffer(yymsp[0].minor.yy41->key, yymsp[-2].minor.yy43);
@@ -860,11 +864,11 @@ static void yy_reduce(
   buffer_free(yymsp[-2].minor.yy43);
   yymsp[-2].minor.yy43 = NULL;
 }
-#line 863 "configparser.c"
+#line 867 "configparser.c"
   yy_destructor(2,&yymsp[-1].minor);
         break;
       case 10:
-#line 172 "./configparser.y"
+#line 172 "../../src/configparser.y"
 {
   array *vars = ctx->current->value;
   data_unset *du;
@@ -904,11 +908,11 @@ static void yy_reduce(
   yymsp[-2].minor.yy43 = NULL;
   yymsp[0].minor.yy41 = NULL;
 }
-#line 907 "configparser.c"
+#line 911 "configparser.c"
   yy_destructor(3,&yymsp[-1].minor);
         break;
       case 11:
-#line 212 "./configparser.y"
+#line 212 "../../src/configparser.y"
 {
   if (strchr(yymsp[0].minor.yy0->ptr, '.') == NULL) {
     yygotominor.yy43 = buffer_init_string("var.");
@@ -920,10 +924,10 @@ static void yy_reduce(
     yymsp[0].minor.yy0 = NULL;
   }
 }
-#line 923 "configparser.c"
+#line 927 "configparser.c"
         break;
       case 12:
-#line 224 "./configparser.y"
+#line 224 "../../src/configparser.y"
 {
   yygotominor.yy41 = configparser_merge_data(yymsp[-2].minor.yy41, yymsp[0].minor.yy41);
   if (NULL == yygotominor.yy41) {
@@ -933,19 +937,19 @@ static void yy_reduce(
   yymsp[0].minor.yy41->free(yymsp[0].minor.yy41);
   yymsp[0].minor.yy41 = NULL;
 }
-#line 936 "configparser.c"
+#line 940 "configparser.c"
   yy_destructor(5,&yymsp[-1].minor);
         break;
       case 13:
-#line 234 "./configparser.y"
+#line 234 "../../src/configparser.y"
 {
   yygotominor.yy41 = yymsp[0].minor.yy41;
   yymsp[0].minor.yy41 = NULL;
 }
-#line 945 "configparser.c"
+#line 949 "configparser.c"
         break;
       case 14:
-#line 239 "./configparser.y"
+#line 239 "../../src/configparser.y"
 {
   yygotominor.yy41 = NULL;
   if (strncmp(yymsp[0].minor.yy43->ptr, "env.", sizeof("env.") - 1) == 0) {
@@ -972,59 +976,59 @@ static void yy_reduce(
   buffer_free(yymsp[0].minor.yy43);
   yymsp[0].minor.yy43 = NULL;
 }
-#line 975 "configparser.c"
+#line 979 "configparser.c"
         break;
       case 15:
-#line 266 "./configparser.y"
+#line 266 "../../src/configparser.y"
 {
   yygotominor.yy41 = (data_unset *)data_string_init();
   buffer_copy_string_buffer(((data_string *)(yygotominor.yy41))->value, yymsp[0].minor.yy0);
   buffer_free(yymsp[0].minor.yy0);
   yymsp[0].minor.yy0 = NULL;
 }
-#line 985 "configparser.c"
+#line 989 "configparser.c"
         break;
       case 16:
-#line 273 "./configparser.y"
+#line 273 "../../src/configparser.y"
 {
   yygotominor.yy41 = (data_unset *)data_integer_init();
   ((data_integer *)(yygotominor.yy41))->value = strtol(yymsp[0].minor.yy0->ptr, NULL, 10);
   buffer_free(yymsp[0].minor.yy0);
   yymsp[0].minor.yy0 = NULL;
 }
-#line 995 "configparser.c"
+#line 999 "configparser.c"
         break;
       case 17:
-#line 279 "./configparser.y"
+#line 279 "../../src/configparser.y"
 {
   yygotominor.yy41 = (data_unset *)data_array_init();
   array_free(((data_array *)(yygotominor.yy41))->value);
   ((data_array *)(yygotominor.yy41))->value = yymsp[0].minor.yy40;
   yymsp[0].minor.yy40 = NULL;
 }
-#line 1005 "configparser.c"
+#line 1009 "configparser.c"
         break;
       case 18:
-#line 285 "./configparser.y"
+#line 285 "../../src/configparser.y"
 {
   yygotominor.yy40 = array_init();
 }
-#line 1012 "configparser.c"
+#line 1016 "configparser.c"
   yy_destructor(8,&yymsp[-1].minor);
   yy_destructor(9,&yymsp[0].minor);
         break;
       case 19:
-#line 288 "./configparser.y"
+#line 288 "../../src/configparser.y"
 {
   yygotominor.yy40 = yymsp[-1].minor.yy40;
   yymsp[-1].minor.yy40 = NULL;
 }
-#line 1022 "configparser.c"
+#line 1026 "configparser.c"
   yy_destructor(8,&yymsp[-2].minor);
   yy_destructor(9,&yymsp[0].minor);
         break;
       case 20:
-#line 293 "./configparser.y"
+#line 293 "../../src/configparser.y"
 {
   if (buffer_is_empty(yymsp[0].minor.yy41->key) ||
       NULL == array_get_element(yymsp[-2].minor.yy40, yymsp[0].minor.yy41->key->ptr)) {
@@ -1041,37 +1045,37 @@ static void yy_reduce(
   yygotominor.yy40 = yymsp[-2].minor.yy40;
   yymsp[-2].minor.yy40 = NULL;
 }
-#line 1044 "configparser.c"
+#line 1048 "configparser.c"
   yy_destructor(10,&yymsp[-1].minor);
         break;
       case 21:
-#line 310 "./configparser.y"
+#line 310 "../../src/configparser.y"
 {
   yygotominor.yy40 = yymsp[-1].minor.yy40;
   yymsp[-1].minor.yy40 = NULL;
 }
-#line 1053 "configparser.c"
+#line 1057 "configparser.c"
   yy_destructor(10,&yymsp[0].minor);
         break;
       case 22:
-#line 315 "./configparser.y"
+#line 315 "../../src/configparser.y"
 {
   yygotominor.yy40 = array_init();
   array_insert_unique(yygotominor.yy40, yymsp[0].minor.yy41);
   yymsp[0].minor.yy41 = NULL;
 }
-#line 1063 "configparser.c"
+#line 1067 "configparser.c"
         break;
       case 23:
-#line 321 "./configparser.y"
+#line 321 "../../src/configparser.y"
 {
   yygotominor.yy41 = yymsp[0].minor.yy41;
   yymsp[0].minor.yy41 = NULL;
 }
-#line 1071 "configparser.c"
+#line 1075 "configparser.c"
         break;
       case 24:
-#line 325 "./configparser.y"
+#line 325 "../../src/configparser.y"
 {
   buffer_copy_string_buffer(yymsp[0].minor.yy41->key, yymsp[-2].minor.yy43);
   buffer_free(yymsp[-2].minor.yy43);
@@ -1080,7 +1084,7 @@ static void yy_reduce(
   yygotominor.yy41 = yymsp[0].minor.yy41;
   yymsp[0].minor.yy41 = NULL;
 }
-#line 1083 "configparser.c"
+#line 1087 "configparser.c"
   yy_destructor(11,&yymsp[-1].minor);
         break;
       case 25:
@@ -1089,18 +1093,18 @@ static void yy_reduce(
       case 26:
         break;
       case 27:
-#line 337 "./configparser.y"
+#line 337 "../../src/configparser.y"
 {
   data_config *dc;
   dc = (data_config *)array_get_element(ctx->srv->config_context, "global");
   assert(dc);
   configparser_push(ctx, dc, 0);
 }
-#line 1099 "configparser.c"
+#line 1103 "configparser.c"
   yy_destructor(12,&yymsp[0].minor);
         break;
       case 28:
-#line 344 "./configparser.y"
+#line 344 "../../src/configparser.y"
 {
   data_config *cur;
 
@@ -1111,14 +1115,14 @@ static void yy_reduce(
 
   yygotominor.yy78 = cur;
 }
-#line 1114 "configparser.c"
+#line 1118 "configparser.c"
         /* No destructor defined for globalstart */
   yy_destructor(13,&yymsp[-2].minor);
         /* No destructor defined for metalines */
   yy_destructor(14,&yymsp[0].minor);
         break;
       case 29:
-#line 355 "./configparser.y"
+#line 355 "../../src/configparser.y"
 {
   assert(yymsp[-3].minor.yy78->context_ndx < yymsp[0].minor.yy78->context_ndx);
   yymsp[0].minor.yy78->prev = yymsp[-3].minor.yy78;
@@ -1127,20 +1131,20 @@ static void yy_reduce(
   yymsp[-3].minor.yy78 = NULL;
   yymsp[0].minor.yy78 = NULL;
 }
-#line 1130 "configparser.c"
+#line 1134 "configparser.c"
         /* No destructor defined for eols */
   yy_destructor(15,&yymsp[-1].minor);
         break;
       case 30:
-#line 364 "./configparser.y"
+#line 364 "../../src/configparser.y"
 {
   yygotominor.yy78 = yymsp[0].minor.yy78;
   yymsp[0].minor.yy78 = NULL;
 }
-#line 1140 "configparser.c"
+#line 1144 "configparser.c"
         break;
       case 31:
-#line 369 "./configparser.y"
+#line 369 "../../src/configparser.y"
 {
   data_config *cur;
 
@@ -1151,14 +1155,14 @@ static void yy_reduce(
 
   yygotominor.yy78 = cur;
 }
-#line 1154 "configparser.c"
+#line 1158 "configparser.c"
         /* No destructor defined for context */
   yy_destructor(13,&yymsp[-2].minor);
         /* No destructor defined for metalines */
   yy_destructor(14,&yymsp[0].minor);
         break;
       case 32:
-#line 380 "./configparser.y"
+#line 380 "../../src/configparser.y"
 {
   data_config *dc;
   buffer *b, *rvalue, *op;
@@ -1209,6 +1213,7 @@ static void yy_reduce(
       { COMP_HTTP_REFERER,       CONST_STR_LEN("HTTP[\"referer\"]"    ) },
       { COMP_HTTP_USER_AGENT,    CONST_STR_LEN("HTTP[\"useragent\"]"  ) },
       { COMP_HTTP_USER_AGENT,    CONST_STR_LEN("HTTP[\"user-agent\"]"  ) },
+      { COMP_HTTP_LANGUAGE,      CONST_STR_LEN("HTTP[\"language\"]"   ) },
       { COMP_HTTP_COOKIE,        CONST_STR_LEN("HTTP[\"cookie\"]"     ) },
       { COMP_HTTP_REMOTE_IP,     CONST_STR_LEN("HTTP[\"remoteip\"]"   ) },
       { COMP_HTTP_REMOTE_IP,     CONST_STR_LEN("HTTP[\"remote-ip\"]"   ) },
@@ -1299,45 +1304,45 @@ static void yy_reduce(
   yymsp[0].minor.yy41->free(yymsp[0].minor.yy41);
   yymsp[0].minor.yy41 = NULL;
 }
-#line 1302 "configparser.c"
+#line 1307 "configparser.c"
   yy_destructor(16,&yymsp[-6].minor);
   yy_destructor(18,&yymsp[-4].minor);
   yy_destructor(19,&yymsp[-2].minor);
         break;
       case 33:
-#line 520 "./configparser.y"
+#line 521 "../../src/configparser.y"
 {
   yygotominor.yy27 = CONFIG_COND_EQ;
 }
-#line 1312 "configparser.c"
+#line 1317 "configparser.c"
   yy_destructor(20,&yymsp[0].minor);
         break;
       case 34:
-#line 523 "./configparser.y"
+#line 524 "../../src/configparser.y"
 {
   yygotominor.yy27 = CONFIG_COND_MATCH;
 }
-#line 1320 "configparser.c"
+#line 1325 "configparser.c"
   yy_destructor(21,&yymsp[0].minor);
         break;
       case 35:
-#line 526 "./configparser.y"
+#line 527 "../../src/configparser.y"
 {
   yygotominor.yy27 = CONFIG_COND_NE;
 }
-#line 1328 "configparser.c"
+#line 1333 "configparser.c"
   yy_destructor(22,&yymsp[0].minor);
         break;
       case 36:
-#line 529 "./configparser.y"
+#line 530 "../../src/configparser.y"
 {
   yygotominor.yy27 = CONFIG_COND_NOMATCH;
 }
-#line 1336 "configparser.c"
+#line 1341 "configparser.c"
   yy_destructor(23,&yymsp[0].minor);
         break;
       case 37:
-#line 533 "./configparser.y"
+#line 534 "../../src/configparser.y"
 {
   yygotominor.yy43 = NULL;
   if (ctx->ok) {
@@ -1354,10 +1359,10 @@ static void yy_reduce(
   yymsp[0].minor.yy41->free(yymsp[0].minor.yy41);
   yymsp[0].minor.yy41 = NULL;
 }
-#line 1357 "configparser.c"
+#line 1362 "configparser.c"
         break;
       case 38:
-#line 550 "./configparser.y"
+#line 551 "../../src/configparser.y"
 {
   if (ctx->ok) {
     if (0 != config_parse_file(ctx->srv, ctx, yymsp[0].minor.yy43->ptr)) {
@@ -1367,11 +1372,11 @@ static void yy_reduce(
     yymsp[0].minor.yy43 = NULL;
   }
 }
-#line 1370 "configparser.c"
+#line 1375 "configparser.c"
   yy_destructor(24,&yymsp[-1].minor);
         break;
       case 39:
-#line 560 "./configparser.y"
+#line 561 "../../src/configparser.y"
 {
   if (ctx->ok) {
     if (0 != config_parse_cmd(ctx->srv, ctx, yymsp[0].minor.yy43->ptr)) {
@@ -1381,7 +1386,7 @@ static void yy_reduce(
     yymsp[0].minor.yy43 = NULL;
   }
 }
-#line 1384 "configparser.c"
+#line 1389 "configparser.c"
   yy_destructor(25,&yymsp[-1].minor);
         break;
   };
@@ -1411,11 +1416,11 @@ static void yy_parse_failed(
   while( yypParser->yyidx>=0 ) yy_pop_parser_stack(yypParser);
   /* Here code is inserted which will be executed whenever the
   ** parser fails */
-#line 110 "./configparser.y"
+#line 110 "../../src/configparser.y"
 
   ctx->ok = 0;
 
-#line 1418 "configparser.c"
+#line 1423 "configparser.c"
   configparserARG_STORE; /* Suppress warning about unused %extra_argument variable */
 }
 
@@ -1427,9 +1432,9 @@ static void yy_syntax_error(
   int yymajor,                   /* The major type of the error token */
   YYMINORTYPE yyminor            /* The minor type of the error token */
 ){
+  configparserARG_FETCH;
   UNUSED(yymajor);
   UNUSED(yyminor);
-  configparserARG_FETCH;
 #define TOKEN (yyminor.yy0)
   configparserARG_STORE; /* Suppress warning about unused %extra_argument variable */
 }
