@@ -192,7 +192,7 @@ static void dump_packet(const unsigned char *data, size_t len) {
 
 static int connection_handle_read_ssl(server *srv, connection *con) {
 #ifdef USE_OPENSSL
-	int r, ssl_err, len;
+	int r, ssl_err, len, count = 0;
 	buffer *b = NULL;
 
 	if (!con->conf.is_ssl) return -1;
@@ -221,13 +221,15 @@ static int connection_handle_read_ssl(server *srv, connection *con) {
 		       	/* we move the buffer to the chunk-queue, no need to free it */
 
 			chunkqueue_append_buffer_weak(con->read_queue, b);
+			count += len;
 			con->bytes_read += len;
 			b = NULL;
 		}
-	} while (len > 0);
+	} while (len > 0 && count < MAX_READ_LIMIT);
 
 
 	if (len < 0) {
+		int oerrno = errno;
 		switch ((r = SSL_get_error(con->ssl, len))) {
 		case SSL_ERROR_WANT_READ:
 		case SSL_ERROR_WANT_WRITE:
@@ -257,11 +259,11 @@ static int connection_handle_read_ssl(server *srv, connection *con) {
 						r, ERR_error_string(ssl_err, NULL));
 			}
 
-			switch(errno) {
+			switch(oerrno) {
 			default:
 				log_error_write(srv, __FILE__, __LINE__, "sddds", "SSL:",
-						len, r, errno,
-						strerror(errno));
+						len, r, oerrno,
+						strerror(oerrno));
 				break;
 			}
 
@@ -330,15 +332,14 @@ static int connection_handle_read(server *srv, connection *con) {
 	buffer_prepare_copy(b, 4 * 1024);
 	len = recv(con->fd, b->ptr, b->size - 1, 0);
 #else
-	if (ioctl(con->fd, FIONREAD, &toread)) {
-		log_error_write(srv, __FILE__, __LINE__, "sd",
-				"unexpected end-of-file:",
-				con->fd);
-		return -1;
+	if (ioctl(con->fd, FIONREAD, &toread) || toread == 0) {
+		b = chunkqueue_get_append_buffer(con->read_queue);
+		buffer_prepare_copy(b, 4 * 1024);
+	} else {
+		if (toread > MAX_READ_LIMIT) toread = MAX_READ_LIMIT;
+		b = chunkqueue_get_append_buffer(con->read_queue);
+		buffer_prepare_copy(b, toread + 1);
 	}
-	b = chunkqueue_get_append_buffer(con->read_queue);
-	buffer_prepare_copy(b, toread + 1);
-
 	len = read(con->fd, b->ptr, b->size - 1);
 #endif
 
@@ -860,7 +861,7 @@ int connection_reset(server *srv, connection *con) {
  *
  * we get called by the state-engine and by the fdevent-handler
  */
-int connection_handle_read_state(server *srv, connection *con)  {
+static int connection_handle_read_state(server *srv, connection *con)  {
 	connection_state_t ostate = con->state;
 	chunk *c, *last_chunk;
 	off_t last_offset;
@@ -1066,6 +1067,9 @@ int connection_handle_read_state(server *srv, connection *con)  {
 						if (dst_c->file.fd == -1) {
 							/* this should not happen as we cache the fd, but you never know */
 							dst_c->file.fd = open(dst_c->file.name->ptr, O_WRONLY | O_APPEND);
+#ifdef FD_CLOEXEC
+							fcntl(dst_c->file.fd, F_SETFD, FD_CLOEXEC);
+#endif
 						}
 					} else {
 						/* the chunk is too large now, close it */
@@ -1155,7 +1159,7 @@ int connection_handle_read_state(server *srv, connection *con)  {
 	return 0;
 }
 
-handler_t connection_handle_fdevent(void *s, void *context, int revents) {
+static handler_t connection_handle_fdevent(void *s, void *context, int revents) {
 	server     *srv = (server *)s;
 	connection *con = context;
 
@@ -1179,7 +1183,7 @@ handler_t connection_handle_fdevent(void *s, void *context, int revents) {
 		/* FIXME: revents = 0x19 still means that we should read from the queue */
 		if (revents & FDEVENT_HUP) {
 			if (con->state == CON_STATE_CLOSE) {
-				con->close_timeout_ts = 0;
+				con->close_timeout_ts = srv->cur_ts - 2;
 			} else {
 				/* sigio reports the wrong event here
 				 *
@@ -1248,7 +1252,7 @@ handler_t connection_handle_fdevent(void *s, void *context, int revents) {
 		} else {
 			/* nothing to read */
 
-			con->close_timeout_ts = 0;
+			con->close_timeout_ts = srv->cur_ts - 2;
 		}
 	}
 
@@ -1600,10 +1604,10 @@ int connection_state_machine(server *srv, connection *con) {
 				} else {
 					/* nothing to read */
 
-					con->close_timeout_ts = 0;
+					con->close_timeout_ts = srv->cur_ts - 2;
 				}
 			} else {
-				con->close_timeout_ts = 0;
+				con->close_timeout_ts = srv->cur_ts - 2;
 			}
 
 			if (srv->cur_ts - con->close_timeout_ts > 1) {
