@@ -156,7 +156,7 @@ INIT_FUNC(mod_accesslog_init) {
 	return p;
 }
 
-int accesslog_parse_format(server *srv, format_fields *fields, buffer *format) {
+static int accesslog_parse_format(server *srv, format_fields *fields, buffer *format) {
 	size_t i, j, k = 0, start = 0;
 
 	if (format->used == 0) return -1;
@@ -169,13 +169,13 @@ int accesslog_parse_format(server *srv, format_fields *fields, buffer *format) {
 				if (fields->size == 0) {
 					fields->size = 16;
 					fields->used = 0;
-					fields->ptr = malloc(fields->size * sizeof(format_fields * ));
+					fields->ptr = malloc(fields->size * sizeof(format_field * ));
 				} else if (fields->used == fields->size) {
 					fields->size += 16;
-					fields->ptr = realloc(fields->ptr, fields->size * sizeof(format_fields * ));
+					fields->ptr = realloc(fields->ptr, fields->size * sizeof(format_field * ));
 				}
 
-				fields->ptr[fields->used] = malloc(sizeof(format_fields));
+				fields->ptr[fields->used] = malloc(sizeof(format_field));
 				fields->ptr[fields->used]->type = FIELD_STRING;
 				fields->ptr[fields->used]->string = buffer_init();
 
@@ -189,10 +189,10 @@ int accesslog_parse_format(server *srv, format_fields *fields, buffer *format) {
 			if (fields->size == 0) {
 				fields->size = 16;
 				fields->used = 0;
-				fields->ptr = malloc(fields->size * sizeof(format_fields * ));
+				fields->ptr = malloc(fields->size * sizeof(format_field * ));
 			} else if (fields->used == fields->size) {
 				fields->size += 16;
-				fields->ptr = realloc(fields->ptr, fields->size * sizeof(format_fields * ));
+				fields->ptr = realloc(fields->ptr, fields->size * sizeof(format_field * ));
 			}
 
 			/* search for the terminating command */
@@ -211,7 +211,7 @@ int accesslog_parse_format(server *srv, format_fields *fields, buffer *format) {
 
 					/* found key */
 
-					fields->ptr[fields->used] = malloc(sizeof(format_fields));
+					fields->ptr[fields->used] = malloc(sizeof(format_field));
 					fields->ptr[fields->used]->type = FIELD_FORMAT;
 					fields->ptr[fields->used]->field = fmap[j].type;
 					fields->ptr[fields->used]->string = NULL;
@@ -258,7 +258,7 @@ int accesslog_parse_format(server *srv, format_fields *fields, buffer *format) {
 
 					/* found key */
 
-					fields->ptr[fields->used] = malloc(sizeof(format_fields));
+					fields->ptr[fields->used] = malloc(sizeof(format_field));
 					fields->ptr[fields->used]->type = FIELD_FORMAT;
 					fields->ptr[fields->used]->field = fmap[j].type;
 					fields->ptr[fields->used]->string = buffer_init();
@@ -291,7 +291,7 @@ int accesslog_parse_format(server *srv, format_fields *fields, buffer *format) {
 
 					/* found key */
 
-					fields->ptr[fields->used] = malloc(sizeof(format_fields));
+					fields->ptr[fields->used] = malloc(sizeof(format_field));
 					fields->ptr[fields->used]->type = FIELD_FORMAT;
 					fields->ptr[fields->used]->field = fmap[j].type;
 					fields->ptr[fields->used]->string = NULL;
@@ -321,13 +321,13 @@ int accesslog_parse_format(server *srv, format_fields *fields, buffer *format) {
 		if (fields->size == 0) {
 			fields->size = 16;
 			fields->used = 0;
-			fields->ptr = malloc(fields->size * sizeof(format_fields * ));
+			fields->ptr = malloc(fields->size * sizeof(format_field * ));
 		} else if (fields->used == fields->size) {
 			fields->size += 16;
-			fields->ptr = realloc(fields->ptr, fields->size * sizeof(format_fields * ));
+			fields->ptr = realloc(fields->ptr, fields->size * sizeof(format_field * ));
 		}
 
-		fields->ptr[fields->used] = malloc(sizeof(format_fields));
+		fields->ptr[fields->used] = malloc(sizeof(format_field));
 		fields->ptr[fields->used]->type = FIELD_STRING;
 		fields->ptr[fields->used]->string = buffer_init();
 
@@ -475,72 +475,8 @@ SETDEFAULTS_FUNC(log_access_open) {
 
 		if (s->access_logfile->used < 2) continue;
 
-		if (s->access_logfile->ptr[0] == '|') {
-#ifdef HAVE_FORK
-			/* create write pipe and spawn process */
-
-			int to_log_fds[2];
-			pid_t pid;
-
-			if (pipe(to_log_fds)) {
-				log_error_write(srv, __FILE__, __LINE__, "ss", "pipe failed: ", strerror(errno));
-				return HANDLER_ERROR;
-			}
-
-			/* fork, execve */
-			switch (pid = fork()) {
-			case 0:
-				/* child */
-
-				close(STDIN_FILENO);
-				dup2(to_log_fds[0], STDIN_FILENO);
-				close(to_log_fds[0]);
-				/* not needed */
-				close(to_log_fds[1]);
-
-				openDevNull(STDERR_FILENO);
-
-				/* we don't need the client socket */
-				for (i = 3; i < 256; i++) {
-					close(i);
-				}
-
-				/* exec the log-process (skip the | )
-				 *
-				 */
-
-				execl("/bin/sh", "sh", "-c", s->access_logfile->ptr + 1, (char *)NULL);
-
-				log_error_write(srv, __FILE__, __LINE__, "sss",
-						"spawning log-process failed: ", strerror(errno),
-						s->access_logfile->ptr + 1);
-
-				exit(-1);
-				break;
-			case -1:
-				/* error */
-				log_error_write(srv, __FILE__, __LINE__, "ss", "fork failed: ", strerror(errno));
-				break;
-			default:
-				close(to_log_fds[0]);
-
-				s->log_access_fd = to_log_fds[1];
-
-				break;
-			}
-#else
-			return -1;
-#endif
-		} else if (-1 == (s->log_access_fd =
-				  open(s->access_logfile->ptr, O_APPEND | O_WRONLY | O_CREAT | O_LARGEFILE, 0644))) {
-
-			log_error_write(srv, __FILE__, __LINE__, "ssb",
-					"opening access-log failed:",
-					strerror(errno), s->access_logfile);
-
+		if (-1 == (s->log_access_fd = open_logfile_or_pipe(srv, s->access_logfile->ptr)))
 			return HANDLER_ERROR;
-		}
-		fcntl(s->log_access_fd, F_SETFD, FD_CLOEXEC);
 
 	}
 
@@ -584,6 +520,9 @@ SIGHUP_FUNC(log_access_cycle) {
 
 				return HANDLER_ERROR;
 			}
+#ifdef FD_CLOEXEC
+			fcntl(s->log_access_fd, F_SETFD, FD_CLOEXEC);
+#endif
 		}
 	}
 
@@ -872,6 +811,7 @@ REQUESTDONE_FUNC(log_access_write) {
 }
 
 
+int mod_accesslog_plugin_init(plugin *p);
 int mod_accesslog_plugin_init(plugin *p) {
 	p->version     = LIGHTTPD_VERSION_ID;
 	p->name        = buffer_init_string("accesslog");
