@@ -332,7 +332,7 @@ SETDEFAULTS_FUNC(mod_proxy_set_defaults) {
 	return HANDLER_GO_ON;
 }
 
-void proxy_connection_close(server *srv, handler_ctx *hctx) {
+static void proxy_connection_close(server *srv, handler_ctx *hctx) {
 	plugin_data *p;
 	connection *con;
 
@@ -356,20 +356,35 @@ void proxy_connection_close(server *srv, handler_ctx *hctx) {
 static int proxy_establish_connection(server *srv, handler_ctx *hctx) {
 	struct sockaddr *proxy_addr;
 	struct sockaddr_in proxy_addr_in;
+#if defined(HAVE_IPV6) && defined(HAVE_INET_PTON)
+	struct sockaddr_in6 proxy_addr_in6;
+#endif
 	socklen_t servlen;
 
 	plugin_data *p    = hctx->plugin_data;
 	data_proxy *host= hctx->host;
 	int proxy_fd       = hctx->fd;
 
-	memset(&proxy_addr, 0, sizeof(proxy_addr));
 
-	proxy_addr_in.sin_family = AF_INET;
-	proxy_addr_in.sin_addr.s_addr = inet_addr(host->host->ptr);
-	proxy_addr_in.sin_port = htons(host->port);
-	servlen = sizeof(proxy_addr_in);
+#if defined(HAVE_IPV6) && defined(HAVE_INET_PTON)
+	if (strstr(host->host->ptr, ":")) {
+		memset(&proxy_addr_in6, 0, sizeof(proxy_addr_in6));
+		proxy_addr_in6.sin6_family = AF_INET6;
+		inet_pton(AF_INET6, host->host->ptr, (char *) &proxy_addr_in6.sin6_addr);
+		proxy_addr_in6.sin6_port = htons(host->port);
+		servlen = sizeof(proxy_addr_in6);
+		proxy_addr = (struct sockaddr *) &proxy_addr_in6;
+	} else
+#endif
+	{
+		memset(&proxy_addr_in, 0, sizeof(proxy_addr_in));
+		proxy_addr_in.sin_family = AF_INET;
+		proxy_addr_in.sin_addr.s_addr = inet_addr(host->host->ptr);
+		proxy_addr_in.sin_port = htons(host->port);
+		servlen = sizeof(proxy_addr_in);
+		proxy_addr = (struct sockaddr *) &proxy_addr_in;
+	}
 
-	proxy_addr = (struct sockaddr *) &proxy_addr_in;
 
 	if (-1 == connect(proxy_fd, proxy_addr, servlen)) {
 		if (errno == EINPROGRESS || errno == EALREADY) {
@@ -395,7 +410,7 @@ static int proxy_establish_connection(server *srv, handler_ctx *hctx) {
 	return 0;
 }
 
-void proxy_set_header(connection *con, const char *key, const char *value) {
+static void proxy_set_header(connection *con, const char *key, const char *value) {
     data_string *ds_dst;
 
     if (NULL == (ds_dst = (data_string *)array_get_unused_element(con->request.headers, TYPE_STRING))) {
@@ -407,7 +422,7 @@ void proxy_set_header(connection *con, const char *key, const char *value) {
     array_insert_unique(con->request.headers, (data_unset *)ds_dst);
 }
 
-void proxy_append_header(connection *con, const char *key, const char *value) {
+static void proxy_append_header(connection *con, const char *key, const char *value) {
     data_string *ds_dst;
 
     if (NULL == (ds_dst = (data_string *)array_get_unused_element(con->request.headers, TYPE_STRING))) {
@@ -432,10 +447,10 @@ static int proxy_create_env(server *srv, handler_ctx *hctx) {
 
 	/* request line */
 	buffer_copy_string(b, get_http_method_name(con->request.http_method));
-	BUFFER_APPEND_STRING_CONST(b, " ");
+	buffer_append_string_len(b, CONST_STR_LEN(" "));
 
 	buffer_append_string_buffer(b, con->request.uri);
-	BUFFER_APPEND_STRING_CONST(b, " HTTP/1.0\r\n");
+	buffer_append_string_len(b, CONST_STR_LEN(" HTTP/1.0\r\n"));
 
 	proxy_append_header(con, "X-Forwarded-For", (char *)inet_ntop_cache_get_ip(srv, &(con->dst_addr)));
 	/* http_host is NOT is just a pointer to a buffer
@@ -454,15 +469,16 @@ static int proxy_create_env(server *srv, handler_ctx *hctx) {
 
 		if (ds->value->used && ds->key->used) {
 			if (buffer_is_equal_string(ds->key, CONST_STR_LEN("Connection"))) continue;
+			if (buffer_is_equal_string(ds->key, CONST_STR_LEN("Proxy-Connection"))) continue;
 
 			buffer_append_string_buffer(b, ds->key);
-			BUFFER_APPEND_STRING_CONST(b, ": ");
+			buffer_append_string_len(b, CONST_STR_LEN(": "));
 			buffer_append_string_buffer(b, ds->value);
-			BUFFER_APPEND_STRING_CONST(b, "\r\n");
+			buffer_append_string_len(b, CONST_STR_LEN("\r\n"));
 		}
 	}
 
-	BUFFER_APPEND_STRING_CONST(b, "\r\n");
+	buffer_append_string_len(b, CONST_STR_LEN("\r\n"));
 
 	hctx->wb->bytes_in += b->used - 1;
 	/* body */
@@ -652,7 +668,7 @@ static int proxy_demux_response(server *srv, handler_ctx *hctx) {
 			buffer_prepare_append(hctx->response, b + 1);
 			hctx->response->used = 1;
 		} else {
-			buffer_prepare_append(hctx->response, hctx->response->used + b);
+			buffer_prepare_append(hctx->response, b);
 		}
 
 		if (-1 == (r = read(hctx->fd, hctx->response->ptr + hctx->response->used - 1, b))) {
@@ -740,9 +756,16 @@ static handler_t proxy_write_request(server *srv, handler_ctx *hctx) {
 
 	switch(hctx->state) {
 	case PROXY_STATE_INIT:
-		if (-1 == (hctx->fd = socket(AF_INET, SOCK_STREAM, 0))) {
+		if (strstr(host->host->ptr,":")) {
+		    if (-1 == (hctx->fd = socket(AF_INET6, SOCK_STREAM, 0))) {
 			log_error_write(srv, __FILE__, __LINE__, "ss", "socket failed: ", strerror(errno));
 			return HANDLER_ERROR;
+		    }
+		} else {
+		    if (-1 == (hctx->fd = socket(AF_INET, SOCK_STREAM, 0))) {
+			log_error_write(srv, __FILE__, __LINE__, "ss", "socket failed: ", strerror(errno));
+			return HANDLER_ERROR;
+		    }
 		}
 		hctx->fde_ndx = -1;
 
@@ -1060,6 +1083,8 @@ static handler_t mod_proxy_check_extension(server *srv, connection *con, void *p
 	data_array *extension = NULL;
 	size_t path_info_offset;
 
+	if (con->mode != DIRECT) return HANDLER_GO_ON;
+
 	/* Possibly, we processed already this request */
 	if (con->file_started == 1) return HANDLER_GO_ON;
 
@@ -1093,15 +1118,17 @@ static handler_t mod_proxy_check_extension(server *srv, connection *con, void *p
 		if (s_len < ct_len) continue;
 
 		/* check extension in the form "/proxy_pattern" */
-		if (*(extension->key->ptr) == '/' && strncmp(fn->ptr, extension->key->ptr, ct_len) == 0) {
-			if (s_len > ct_len + 1) {
-				char *pi_offset;
+		if (*(extension->key->ptr) == '/') {
+			if (strncmp(fn->ptr, extension->key->ptr, ct_len) == 0) {
+				if (s_len > ct_len + 1) {
+					char *pi_offset;
 
-				if (0 != (pi_offset = strchr(fn->ptr + ct_len + 1, '/'))) {
-					path_info_offset = pi_offset - fn->ptr;
+					if (0 != (pi_offset = strchr(fn->ptr + ct_len + 1, '/'))) {
+						path_info_offset = pi_offset - fn->ptr;
+					}
 				}
+				break;
 			}
-			break;
 		} else if (0 == strncmp(fn->ptr + s_len - ct_len, extension->key->ptr, ct_len)) {
 			/* check extension in the form ".fcg" */
 			break;
@@ -1116,7 +1143,13 @@ static handler_t mod_proxy_check_extension(server *srv, connection *con, void *p
 		log_error_write(srv, __FILE__, __LINE__,  "s", "proxy - ext found");
 	}
 
-	switch(p->conf.balance) {
+	if (extension->value->used == 1) {
+		if ( ((data_proxy *)extension->value->data[0])->is_disabled ) {
+			ndx = -1;
+		} else {
+			ndx = 0;
+		}
+	} else if (extension->value->used != 0) switch(p->conf.balance) {
 	case PROXY_BALANCE_HASH:
 		/* hash balancing */
 
@@ -1173,7 +1206,9 @@ static handler_t mod_proxy_check_extension(server *srv, connection *con, void *p
 		}
 
 		break;
-	case PROXY_BALANCE_RR:
+	case PROXY_BALANCE_RR: {
+		data_proxy *host;
+
 		/* round robin */
 		if (p->conf.debug) {
 			log_error_write(srv, __FILE__, __LINE__,  "s",
@@ -1183,31 +1218,33 @@ static handler_t mod_proxy_check_extension(server *srv, connection *con, void *p
 		/* just to be sure */
 		assert(extension->value->used < INT_MAX);
 
-		for (k = 0, ndx = -1, max_usage = INT_MAX; k < extension->value->used; k++) {
-			data_proxy *host = (data_proxy *)extension->value->data[k];
+		host = (data_proxy *)extension->value->data[0];
 
-			if (host->is_disabled) continue;
+		/* Use last_used_ndx from first host in list */
+		k = host->last_used_ndx;
+		ndx = k + 1; /* use next host after the last one */
+		if (ndx < 0) ndx = 0;
 
-			/* first usable ndx */
-			if (max_usage == INT_MAX) {
-				max_usage = k;
+		/* Search first active host after last_used_ndx */
+		while ( ndx < (int) extension->value->used
+				&& (host = (data_proxy *)extension->value->data[ndx])->is_disabled ) ndx++;
+
+		if (ndx >= (int) extension->value->used) {
+			/* didn't found a higher id, wrap to the start */
+			for (ndx = 0; ndx <= (int) k; ndx++) {
+				host = (data_proxy *)extension->value->data[ndx];
+				if (!host->is_disabled) break;
 			}
 
-			/* get next ndx */
-			if ((int)k > host->last_used_ndx) {
-				ndx = k;
-				host->last_used_ndx = k;
-
-				break;
-			}
+			/* No active host found */
+			if (host->is_disabled) ndx = -1;
 		}
 
-		/* didn't found a higher id, wrap to the start */
-		if (ndx != -1 && max_usage != INT_MAX) {
-			ndx = max_usage;
-		}
+		/* Save new index for next round */
+		((data_proxy *)extension->value->data[0])->last_used_ndx = ndx;
 
 		break;
+	}
 	default:
 		break;
 	}
@@ -1306,6 +1343,7 @@ TRIGGER_FUNC(mod_proxy_trigger) {
 }
 
 
+int mod_proxy_plugin_init(plugin *p);
 int mod_proxy_plugin_init(plugin *p) {
 	p->version      = LIGHTTPD_VERSION_ID;
 	p->name         = buffer_init_string("proxy");
